@@ -1,49 +1,27 @@
-#!/usr/bin/env python2
 """
 Module for creating Sankey diagrams using matplotlib
 """
-from __future__ import print_function
-__author__ = "Kevin L. Davies"
-__credits__ = ["Yannick Copin"]
-__license__ = "BSD"
-__version__ = "2011/09/16"
-
-# Original version by Yannick Copin (ycopin@ipnl.in2p3.fr) 10/2/2010, available
-# at:
-#     http://matplotlib.org/examples/api/sankey_demo_old.html
-# Modifications by Kevin Davies (kld@alumni.carnegiemellon.edu) 6/3/2011:
-#   --Used arcs for the curves (so that the widths of the paths are uniform)
-#   --Converted the function to a class and created methods to join multiple
-#     simple Sankey diagrams
-#   --Provided handling for cases where the total of the inputs isn't 100
-#     Now, the default layout is based on the assumption that the inputs sum to
-#     1.  A scaling parameter can be used in other cases.
-#   --The call structure was changed to be more explicit about layout,
-#     including the length of the trunk, length of the paths, gap between the
-#     paths, and the margin around the diagram.
-#   --Allowed the lengths of paths to be adjusted individually, with an option
-#     to automatically justify them
-#   --The call structure was changed to make the specification of path
-#     orientation more flexible.  Flows are passed through one array, with
-#     inputs being positive and outputs being negative.  An orientation
-#     argument specifies the direction of the arrows.  The "main"
-#     inputs/outputs are now specified via an orientation of 0, and there may
-#     be several of each.
-#   --Added assertions to catch common calling errors
-#   --Added the physical unit as a string argument to be used in the labels, so
-#     that the values of the flows can usually be applied automatically
-#   --Added an argument for a minimum magnitude below which flows are not shown
-#   --Added a tapered trunk in the case that the flows do not sum to 0
-#   --Allowed the diagram to be rotated
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
 
+import six
+import logging
+from six.moves import zip
 import numpy as np
 
 from matplotlib.cbook import iterable, Bunch
 from matplotlib.path import Path
 from matplotlib.patches import PathPatch
 from matplotlib.transforms import Affine2D
-from matplotlib import verbose
 from matplotlib import docstring
+from matplotlib import rcParams
+
+_log = logging.getLogger(__name__)
+
+__author__ = "Kevin L. Davies"
+__credits__ = ["Yannick Copin"]
+__license__ = "BSD"
+__version__ = "2011/09/16"
 
 # Angles [deg/90]
 RIGHT = 0
@@ -52,7 +30,7 @@ UP = 1
 DOWN = 3
 
 
-class Sankey:
+class Sankey(object):
     """
     Sankey diagram in matplotlib
 
@@ -60,7 +38,7 @@ class Sankey:
       the width of the arrows is shown proportionally to the flow
       quantity.  They are typically used to visualize energy or
       material or cost transfers between processes.
-      `Wikipedia (6/1/2011) <http://en.wikipedia.org/wiki/Sankey_diagram>`_
+      `Wikipedia (6/1/2011) <https://en.wikipedia.org/wiki/Sankey_diagram>`_
 
     """
 
@@ -142,20 +120,24 @@ class Sankey:
 
         **Examples:**
 
-            .. plot:: mpl_examples/api/sankey_demo_basics.py
+            .. plot:: gallery/api/sankey_basics.py
         """
         # Check the arguments.
-        assert gap >= 0, (
+        if gap < 0:
+            raise ValueError(
             "The gap is negative.\nThis isn't allowed because it "
             "would cause the paths to overlap.")
-        assert radius <= gap, (
+        if radius > gap:
+            raise ValueError(
             "The inner radius is greater than the path spacing.\n"
             "This isn't allowed because it would cause the paths to overlap.")
-        assert head_angle >= 0, (
+        if head_angle < 0:
+            raise ValueError(
             "The angle is negative.\nThis isn't allowed "
             "because it would cause inputs to look like "
             "outputs and vice versa.")
-        assert tolerance >= 0, (
+        if tolerance < 0:
+            raise ValueError(
             "The tolerance is negative.\nIt must be a magnitude.")
 
         # Create axes if necessary.
@@ -220,7 +202,7 @@ class Sankey:
                                  [5.19642327e-01, 8.94571235e-01],
                                  [2.65114773e-01, 1.00000000e+00],
                                  # Insignificant
-                                 #[6.12303177e-17, 1.00000000e+00]])
+                                 # [6.12303177e-17, 1.00000000e+00]])
                                  [0.00000000e+00, 1.00000000e+00]])
         if quadrant == 0 or quadrant == 2:
             if cw:
@@ -238,8 +220,8 @@ class Sankey:
                                              ARC_VERTICES[:, 1]))
         if quadrant > 1:
             radius = -radius  # Rotate 180 deg.
-        return zip(ARC_CODES, radius * vertices +
-                              np.tile(center, (ARC_VERTICES.shape[0], 1)))
+        return list(zip(ARC_CODES, radius * vertices +
+                        np.tile(center, (ARC_VERTICES.shape[0], 1))))
 
     def _add_input(self, path, angle, flow, length):
         """
@@ -364,10 +346,10 @@ class Sankey:
         return reverse_path
         # This might be more efficient, but it fails because 'tuple' object
         # doesn't support item assignment:
-        #path[1] = path[1][-1:0:-1]
-        #path[1][0] = first_action
-        #path[2] = path[2][::-1]
-        #return path
+        # path[1] = path[1][-1:0:-1]
+        # path[1][0] = first_action
+        # path[2] = path[2][::-1]
+        # return path
 
     @docstring.dedent_interpd
     def add(self, patchlabel='', flows=None, orientations=None, labels='',
@@ -444,7 +426,7 @@ class Sankey:
         placed along the sides of the diagram from the top down and along the
         bottom from the outside in.
 
-        If the the sum of the inputs and outputs is nonzero, the discrepancy
+        If the sum of the inputs and outputs is nonzero, the discrepancy
         will appear as a cubic Bezier curve along the top and bottom edges of
         the trunk.
 
@@ -465,62 +447,71 @@ class Sankey:
             rotation /= 90.0
         if orientations is None:
             orientations = [0, 0]
-        assert len(orientations) == n, (
+        if len(orientations) != n:
+            raise ValueError(
             "orientations and flows must have the same length.\n"
             "orientations has length %d, but flows has length %d."
             % (len(orientations), n))
         if labels != '' and getattr(labels, '__iter__', False):
-        # iterable() isn't used because it would give True if labels is a
-        # string
-            assert len(labels) == n, (
+            # iterable() isn't used because it would give True if labels is a
+            # string
+            if len(labels) != n:
+                raise ValueError(
                 "If labels is a list, then labels and flows must have the "
                 "same length.\nlabels has length %d, but flows has length %d."
                 % (len(labels), n))
         else:
             labels = [labels] * n
-        assert trunklength >= 0, (
+        if trunklength < 0:
+            raise ValueError(
             "trunklength is negative.\nThis isn't allowed, because it would "
             "cause poor layout.")
-        if np.absolute(np.sum(flows)) > self.tolerance:
-            verbose.report(
-                "The sum of the flows is nonzero (%f).\nIs the "
-                "system not at steady state?" % np.sum(flows), 'helpful')
+        if np.abs(np.sum(flows)) > self.tolerance:
+            _log.info("The sum of the flows is nonzero (%f).\nIs the "
+                     "system not at steady state?", np.sum(flows))
         scaled_flows = self.scale * flows
         gain = sum(max(flow, 0) for flow in scaled_flows)
         loss = sum(min(flow, 0) for flow in scaled_flows)
         if not (0.5 <= gain <= 2.0):
-            verbose.report(
+            _log.info(
                 "The scaled sum of the inputs is %f.\nThis may "
                 "cause poor layout.\nConsider changing the scale so"
-                " that the scaled sum is approximately 1.0." % gain, 'helpful')
+                " that the scaled sum is approximately 1.0.", gain)
         if not (-2.0 <= loss <= -0.5):
-            verbose.report(
+            _log.info(
                 "The scaled sum of the outputs is %f.\nThis may "
                 "cause poor layout.\nConsider changing the scale so"
-                " that the scaled sum is approximately 1.0." % gain, 'helpful')
+                " that the scaled sum is approximately 1.0.", gain)
         if prior is not None:
-            assert prior >= 0, "The index of the prior diagram is negative."
-            assert min(connect) >= 0, (
+            if prior < 0:
+                raise ValueError("The index of the prior diagram is negative.")
+            if min(connect) < 0:
+                raise ValueError(
                 "At least one of the connection indices is negative.")
-            assert prior < len(self.diagrams), (
+            if prior >= len(self.diagrams):
+                raise ValueError(
                 "The index of the prior diagram is %d, but there are "
                 "only %d other diagrams.\nThe index is zero-based."
                 % (prior, len(self.diagrams)))
-            assert connect[0] < len(self.diagrams[prior].flows), (
+            if connect[0] >= len(self.diagrams[prior].flows):
+                raise ValueError(
                 "The connection index to the source diagram is %d, but "
                 "that diagram has only %d flows.\nThe index is zero-based."
                 % (connect[0], len(self.diagrams[prior].flows)))
-            assert connect[1] < n, (
+            if connect[1] >= n:
+                raise ValueError(
                 "The connection index to this diagram is %d, but this diagram"
                 "has only %d flows.\n The index is zero-based."
                 % (connect[1], n))
-            assert self.diagrams[prior].angles[connect[0]] is not None, (
+            if self.diagrams[prior].angles[connect[0]] is None:
+                raise ValueError(
                 "The connection cannot be made.  Check that the magnitude "
                 "of flow %d of diagram %d is greater than or equal to the "
                 "specified tolerance." % (connect[0], prior))
             flow_error = (self.diagrams[prior].flows[connect[0]] +
                           flows[connect[1]])
-            assert abs(flow_error) < self.tolerance, (
+            if abs(flow_error) >= self.tolerance:
+                raise ValueError(
                 "The scaled sum of the connected flows is %f, which is not "
                 "within the tolerance (%f)." % (flow_error, self.tolerance))
 
@@ -532,11 +523,11 @@ class Sankey:
             elif flow <= -self.tolerance:
                 are_inputs[i] = False
             else:
-                verbose.report(
+                _log.info(
                     "The magnitude of flow %d (%f) is below the "
                     "tolerance (%f).\nIt will not be shown, and it "
                     "cannot be used in a connection."
-                    % (i, flow, self.tolerance), 'helpful')
+                    % (i, flow, self.tolerance))
 
         # Determine the angles of the arrows (before rotation).
         angles = [None] * n
@@ -551,9 +542,10 @@ class Sankey:
                 if is_input is not None:
                     angles[i] = RIGHT
             else:
-                assert orient == -1, (
+                if orient != -1:
+                    raise ValueError(
                     "The value of orientations[%d] is %d, "
-                    "but it must be -1, 0, or 1." % (i, orient))
+                    "but it must be [ -1 | 0 | 1 ]." % (i, orient))
                 if is_input:
                     angles[i] = UP
                 elif not is_input:
@@ -561,7 +553,8 @@ class Sankey:
 
         # Justify the lengths of the paths.
         if iterable(pathlengths):
-            assert len(pathlengths) == n, (
+            if len(pathlengths) != n:
+                raise ValueError(
                 "If pathlengths is a list, then pathlengths and flows must "
                 "have the same length.\npathlengths has length %d, but flows "
                 "has length %d." % (len(pathlengths), n))
@@ -584,8 +577,8 @@ class Sankey:
                     urlength -= flow  # Flow is negative for outputs.
             # Determine the lengths of the bottom-side arrows
             # from the middle outwards.
-            for i, (angle, is_input, flow) in enumerate(reversed(zip(
-                angles, are_inputs, scaled_flows))):
+            for i, (angle, is_input, flow) in enumerate(reversed(list(zip(
+                  angles, are_inputs, scaled_flows)))):
                 if angle == UP and is_input:
                     pathlengths[n - i - 1] = lllength
                     lllength += flow
@@ -595,8 +588,8 @@ class Sankey:
             # Determine the lengths of the left-side arrows
             # from the bottom upwards.
             has_left_input = False
-            for i, (angle, is_input, spec) in enumerate(reversed(zip(
-                angles, are_inputs, zip(scaled_flows, pathlengths)))):
+            for i, (angle, is_input, spec) in enumerate(reversed(list(zip(
+                  angles, are_inputs, zip(scaled_flows, pathlengths))))):
                 if angle == RIGHT:
                     if is_input:
                         if has_left_input:
@@ -607,7 +600,7 @@ class Sankey:
             # from the top downwards.
             has_right_output = False
             for i, (angle, is_input, spec) in enumerate(zip(
-                angles, are_inputs, zip(scaled_flows, pathlengths))):
+                  angles, are_inputs, list(zip(scaled_flows, pathlengths)))):
                 if angle == RIGHT:
                     if not is_input:
                         if has_right_output:
@@ -651,7 +644,7 @@ class Sankey:
         label_locations = np.zeros((n, 2))
         # Add the top-side inputs and outputs from the middle outwards.
         for i, (angle, is_input, spec) in enumerate(zip(
-            angles, are_inputs, zip(scaled_flows, pathlengths))):
+              angles, are_inputs, list(zip(scaled_flows, pathlengths)))):
             if angle == DOWN and is_input:
                 tips[i, :], label_locations[i, :] = self._add_input(
                     ulpath, angle, *spec)
@@ -659,8 +652,8 @@ class Sankey:
                 tips[i, :], label_locations[i, :] = self._add_output(
                     urpath, angle, *spec)
         # Add the bottom-side inputs and outputs from the middle outwards.
-        for i, (angle, is_input, spec) in enumerate(reversed(zip(
-            angles, are_inputs, zip(scaled_flows, pathlengths)))):
+        for i, (angle, is_input, spec) in enumerate(reversed(list(zip(
+              angles, are_inputs, list(zip(scaled_flows, pathlengths)))))):
             if angle == UP and is_input:
                 tip, label_location = self._add_input(llpath, angle, *spec)
                 tips[n - i - 1, :] = tip
@@ -671,8 +664,8 @@ class Sankey:
                 label_locations[n - i - 1, :] = label_location
         # Add the left-side inputs from the bottom upwards.
         has_left_input = False
-        for i, (angle, is_input, spec) in enumerate(reversed(zip(
-            angles, are_inputs, zip(scaled_flows, pathlengths)))):
+        for i, (angle, is_input, spec) in enumerate(reversed(list(zip(
+              angles, are_inputs, list(zip(scaled_flows, pathlengths)))))):
             if angle == RIGHT and is_input:
                 if not has_left_input:
                     # Make sure the lower path extends
@@ -687,7 +680,7 @@ class Sankey:
         # Add the right-side outputs from the top downwards.
         has_right_output = False
         for i, (angle, is_input, spec) in enumerate(zip(
-            angles, are_inputs, zip(scaled_flows, pathlengths))):
+              angles, are_inputs, list(zip(scaled_flows, pathlengths)))):
             if angle == RIGHT and not is_input:
                 if not has_right_output:
                     # Make sure the upper path extends
@@ -723,7 +716,7 @@ class Sankey:
         if prior is None:
             if rotation != 0:  # By default, none of this is needed.
                 angles = [_get_angle(angle, rotation) for angle in angles]
-                rotate = Affine2D().rotate_deg(rotation * 90).transform_point
+                rotate = Affine2D().rotate_deg(rotation * 90).transform_affine
                 tips = rotate(tips)
                 label_locations = rotate(label_locations)
                 vertices = rotate(vertices)
@@ -732,27 +725,24 @@ class Sankey:
             rotation = (self.diagrams[prior].angles[connect[0]] -
                         angles[connect[1]])
             angles = [_get_angle(angle, rotation) for angle in angles]
-            rotate = Affine2D().rotate_deg(rotation * 90).transform_point
+            rotate = Affine2D().rotate_deg(rotation * 90).transform_affine
             tips = rotate(tips)
             offset = self.diagrams[prior].tips[connect[0]] - tips[connect[1]]
-            translate = Affine2D().translate(*offset).transform_point
+            translate = Affine2D().translate(*offset).transform_affine
             tips = translate(tips)
             label_locations = translate(rotate(label_locations))
             vertices = translate(rotate(vertices))
             kwds = dict(s=patchlabel, ha='center', va='center')
             text = self.ax.text(*offset, **kwds)
-        if False:  # Debug
-            print("llpath\n", llpath)
-            print("ulpath\n", self._revert(ulpath))
-            print("urpath\n", urpath)
-            print("lrpath\n", self._revert(lrpath))
-            xs, ys = zip(*vertices)
-            self.ax.plot(xs, ys, 'go-')
-        patch = PathPatch(Path(vertices, codes),
-                          fc=kwargs.pop('fc', kwargs.pop('facecolor',
-                                        '#bfd1d4')),  # Custom defaults
-                          lw=kwargs.pop('lw', kwargs.pop('linewidth', 0.5)),
-                          **kwargs)
+        if rcParams['_internal.classic_mode']:
+            fc = kwargs.pop('fc', kwargs.pop('facecolor', '#bfd1d4'))
+            lw = kwargs.pop('lw', kwargs.pop('linewidth', 0.5))
+        else:
+            fc = kwargs.pop('fc', kwargs.pop('facecolor', None))
+            lw = kwargs.pop('lw', kwargs.pop('linewidth', None))
+        if fc is None:
+            fc = next(self.ax._get_patches_for_fill.prop_cycler)['color']
+        patch = PathPatch(Path(vertices, codes), fc=fc, lw=lw, **kwargs)
         self.ax.add_patch(patch)
 
         # Add the path labels.
